var _ = require('../util')
var keyFilter = require('../filters').key

module.exports = {

  acceptStatement: true,
  priority: 700,

  bind: function () {
    // 1.0.0 key filter
    var rawArg = this.arg
    var keyIndex = rawArg.indexOf('.')
    if (keyIndex > -1) {
      this.arg = rawArg.slice(0, keyIndex)
      this.key = rawArg.slice(keyIndex + 1)
    }

    // warn old usage
    if (process.env.NODE_ENV !== 'production') {
      if (this.filters) {
        var hasKeyFilter = this.filters.some(function (f) {
          return f.name === 'key'
        })
        if (hasKeyFilter) {
          _.deprecation.KEY_FILTER()
        }
      }
    }

    // deal with iframes
    if (
      this.el.tagName === 'IFRAME' &&
      this.arg !== 'load'
    ) {
      var self = this
      this.iframeBind = function () {
        _.on(self.el.contentWindow, self.arg, self.handler)
      }
      this.on('load', this.iframeBind)
    }
  },

  update: function (handler) {
    if (typeof handler !== 'function') {
      process.env.NODE_ENV !== 'production' && _.warn(
        'Directive v-on="' + this.arg + ': ' +
        this.expression + '" expects a function value, ' +
        'got ' + handler
      )
      return
    }

    if (this.key) {
      handler = keyFilter(handler, this.key)
    }

    this.reset()
    var scope = this._scope || this.vm
    this.handler = function (e) {
      scope.$event = e
      var res = handler(e)
      scope.$event = null
      return res
    }
    if (this.iframeBind) {
      this.iframeBind()
    } else {
      _.on(this.el, this.arg, this.handler)
    }
  },

  reset: function () {
    var el = this.iframeBind
      ? this.el.contentWindow
      : this.el
    if (this.handler) {
      _.off(el, this.arg, this.handler)
    }
  },

  unbind: function () {
    this.reset()
  }
}
